#include <math.h>
#include <stdio.h>
#include <string.h>
#include <tcl.h>
#include <libexif/exif-data.h>
#include "exiftcl.h"

/* Remove spaces on the right of the string */
static void trim_spaces(char *buf)
{
    char *s = buf-1;
    for (; *buf; ++buf) {
        if (*buf != ' ')
            s = buf;
    }
    *++s = 0; /* nul terminate the string on the first of the final spaces */
}

/* Show the tag name and contents if the tag exists */
static int getTag(Tcl_Interp *interp, ExifData *d, ExifIfd ifd, ExifTag tag)
{
  char buf[1024];

  buf[0] = '\0';
  /* See if this tag exists */
  ExifEntry *entry = exif_content_get_entry(d->ifd[ifd],tag);
  if (entry) {
    /* Get the contents of the tag in human-readable form */
    exif_entry_get_value(entry, buf, sizeof(buf));

    /* Don't bother printing it if it's entirely blank */
    trim_spaces(buf);
    if (*buf) {
      DBG(1, ("%s: %s\n", exif_tag_get_name_in_ifd(tag,ifd), buf));
    }
  }
  Tcl_SetObjResult(interp, Tcl_NewStringObj(buf, -1));
  return TCL_OK;
}

int tagCmd(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
{
  char *filename;
  ExifData *ed;
  static const char* tags[] = {
    "timestamp", NULL
  };
  enum tagsIdx {
    TIMESTAMP_IX
  };
  int index;

  if (objc < 4) {
    Tcl_WrongNumArgs(interp, 1, objv, "filename tag");
    return TCL_ERROR;
  }
  if (Tcl_GetIndexFromObj(interp, objv[3], tags, "option", 0, &index)
      != TCL_OK) {
    return TCL_ERROR;
  }
  filename = Tcl_GetStringFromObj(objv[2], NULL);
  ed = exif_data_new_from_file(filename);
  if (!ed) {
    Tcl_SetObjResult(interp, Tcl_NewStringObj("", -1));
    return TCL_OK;
  }
  switch (index) {
    case TIMESTAMP_IX:
      return getTag(interp, ed, EXIF_IFD_EXIF, EXIF_TAG_DATE_TIME_ORIGINAL);
  }
  return TCL_OK;
}

int ExifCmd(ClientData data, Tcl_Interp *interp, int objc,
    Tcl_Obj *CONST objv[])
{
  static const char* subCmds[] = {
    "tag", NULL
  };
  enum surfaceCmdIdx {
    TAG_IX
  };
  int index;

  if (objc < 2) {
    Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
    return TCL_ERROR;
  }
  if (Tcl_GetIndexFromObj(interp, objv[1], subCmds, "option", 0, &index)
      != TCL_OK) {
    return TCL_ERROR;
  }
  switch (index) {
    case TAG_IX:
      return tagCmd(interp, objc, objv);
      break;
  }
  return TCL_OK;
}

int Exiftcl_Init(Tcl_Interp *interp) {
  struct SurfaceState *statePtr;

  if (Tcl_InitStubs(interp, "8.0", 0) == NULL) {
    return TCL_ERROR;
  }
  if (Tcl_PkgRequire(interp, "Tcl", "8.0", 0) == NULL) {
    return TCL_ERROR;
  }
  Tcl_CreateObjCommand(interp, "exif", ExifCmd,
    (ClientData)statePtr, NULL);
  if (Tcl_PkgProvide(interp, "exif", EXIFTCL_VERSION) != TCL_OK) {
    return TCL_ERROR;
  }
  return TCL_OK;
}

